import std.stream;
import std.string;
import std.random;
import Vec3D;

//int main(char[][] args)
//{
//  ReadOBJ robj = new ReadOBJ() ;
//
//  robj.open("test.obj") ;
//  robj.read() ;
//
//  return(0) ;
//}

struct Particle{/** 粒子構造体 **/
  vec3D  p    ; /** 位置   **/
  vec3D  v    ; /** 速度   **/
  vec3D  a    ; /** 加速度 **/
  vec3D  f    ; /** 力     **/
  double m    ; /** 質量   **/
  int    lock ; /** 動かせない頂点=1 **/
}

struct Spring{/** バネ構造体 **/
  int    p0 ; /** 粒子インデックス 1 **/
  int    p1 ; /** 粒子インデックス 2 **/
  double k  ; /** バネ定数 **/
  double d  ; /** 減衰定数 **/
  double l  ; /** 無負荷時のバネの長さ **/
}

class Cloth {
  Particle[] particle ;
  Spring[]   spring ;
  int width  ;
  int height ;
  
  this(){
    int x,y,i ;
    
    width=10 ; height=10 ;
    
    particle.length=width*height ;

    for( x=0,y=0 ; x<width ; x++ ){
      particle[y*width+x].lock=1 ;
      particle[y*width+x].v.set(0.0) ;
      particle[y*width+x].a.set(0.0) ;
    }
    
    for( y=0 ; y<height ; y++ ){
      for( x=0 ; x<width ; x++ ){
	particle[y*width+x].p.x= 0.5 * x ;
	particle[y*width+x].p.y= 0.5 * y ;
	particle[y*width+x].p.z= 0.0 ;
	particle[y*width+x].m  = 0.09 ;
      }
    }

    spring.length=((width-1) * (height  ))+
                  ((width+0) * (height-1))+
                  ((width-1) * (height-1))*2 ;
    i=0 ;
    for( y=0 ; y<height ; y++ ){
      for( x=0 ; x<(width-1) ; x++ ){
	spring[i].p0=y*width+x   ;
	spring[i].p1=y*width+x+1 ;
	spring[i].k =0.28 ;
	spring[i].d =0.21 ;
	i++ ;
      }
    }
    for( y=0 ; y<(height-1) ; y++ ){
      for( x=0 ; x<width ; x++ ){
	spring[i].p0=y    *width+x ;
	spring[i].p1=(y+1)*width+x ;
	spring[i].k =0.28 ;
	spring[i].d =0.21 ;
	i++ ;
      }
    }
    for( y=0 ; y<(height-1) ; y++ ){
      for( x=0 ; x<(width-1) ; x++ ){
	spring[i].p0=y    *width+x     ;
	spring[i].p1=(y+1)*width+(x+1) ;
	spring[i].k =0.28 ;
	spring[i].d =0.21 ;
	i++ ;
	spring[i].p0=y    *width+(x+1) ;
	spring[i].p1=(y+1)*width+x     ;
	spring[i].k =0.28 ;
	spring[i].d =0.21 ;
	i++ ;
      }
    }
    
    for( i=0 ; i<spring.length ; i++ ){
      vec3D tmp ;
      tmp.set(particle[spring[i].p0].p) ;
      tmp.sub(particle[spring[i].p1].p) ;
      spring[i].l = tmp.length() ;
    }
  }

  int calc(double dt){
    int i ;
    vec3D axis_y ;
    
    axis_y.x=0 ; axis_y.y=1.0 ; axis_y.z=0 ;

    for( i=0 ; i<particle.length ; i++ ){
      particle[i].f.set(0.0) ;
    }

    for( i=0 ; i<particle.length ; i++ ){ /** 適当な風 **/
      particle[i].f.x = -0.002 * particle[i].m ;
      particle[i].f.rotate( cast(double)((rand()%65536))*4/65536.0 , axis_y ) ;
    }

    for( i=0 ; i<particle.length ; i++ ){ /** 重力 **/
      particle[i].f.y = -0.0098 * particle[i].m ;
    }

    for( i=0 ; i<spring.length ; i++ ){ /** 頂点を結ぶバネから頂点にかかる力を積分 **/
      int p0,p1 ;
      vec3D v,d ;
      vec3D f0,f1 ;
      vec3D tmp ;
      double L,t ;
      
      p0=spring[i].p0 ;
      p1=spring[i].p1 ;
      d.set(particle[p0].p) ; d.sub(particle[p1].p);
      v.set(particle[p0].v) ; v.sub(particle[p1].v);
      L = spring[i].l ;
      f0.set(d) ; f0.normalize() ;
      if( d.length() > 0.01 ){
	t= - ( spring[i].k * (d.length() - L) + (spring[i].d * ( v.dot(d)/d.length() ) ) ) ;
      }else{
	t= - ( spring[i].k * (d.length() - L)) ;
      }
	
      f0.scale( t ) ;
      //      if( i==0 ){
      //	printf("p0=%f,%f,%f p1=%f,%f,%f fl=%f d=%f\n",
      //	       particle[p0].p.x,particle[p0].p.y,particle[p0].p.z,
      //	       particle[p1].p.x,particle[p1].p.y,particle[p1].p.z,
      //	       f0.length(),d.length()) ;
      //      }
      f1.set(f0) ; f1.inv() ;
      if( particle[p0].lock == 0 ){
	particle[p0].f.add(f0) ;
      }
      if( particle[p1].lock == 0 ){
	particle[p1].f.add(f1) ;
      }
    }

    for( i=0 ; i<particle.length ; i++ ){ /** 微小時間dtにおける位置と速度を積分 **/
      if( particle[i].lock==1 ) continue ;
      vec3D Ae ;
      vec3D V ;
      vec3D D ;
      
      Ae.set(particle[i].f) ; Ae.scale(dt/particle[i].m) ;
      particle[i].a.set(Ae) ;
      V.set(Ae) ; V.scale(dt) ;
      particle[i].v.add(V) ;
      D.set(particle[i].v) ; D.scale(dt) ;
      particle[i].p.add(D) ;
    }
    
    return(0) ;
  }
  
}
