import win32.windows;
import win32frame.Frame;
import GLUW ;
import keystat ;

ushort LOWORD(uint l){ return cast(ushort)(l & 0xffff); }
ushort HIWORD(uint l){ return cast(ushort)(l >>> 16)  ; }


class OpenGLFrame : Frame {
  HINSTANCE hInstance ;
  HGLRC hRC;
  HDC   WinDC;
  static PIXELFORMATDESCRIPTOR pfd = { 
    PIXELFORMATDESCRIPTOR.sizeof,    // size of this pfd 
    1,                               // version number 
    PFD_DRAW_TO_WINDOW |             // support window 
    PFD_SUPPORT_OPENGL |             // support OpenGL 
    PFD_DOUBLEBUFFER   |             // double buffered 
    PFD_TYPE_RGBA,                   // RGBA type 
    24,                              // 24-bit color depth 
    0, 0, 0, 0, 0, 0,                // color bits ignored 
    0,                               // no alpha buffer 
    0,                               // shift bit ignored 
    0,                               // no accumulation buffer 
    0, 0, 0, 0,                      // accum bits ignored 
    32,                              // 32-bit z-buffer 
    0,                               // no stencil buffer 
    0,                               // no auxiliary buffer 
    PFD_MAIN_PLANE,                  // main layer 
    0,                               // reserved 
    0, 0, 0                          // layer masks ignored 
  }; 
  int pixelFormat;
  GLUW gluw ;
  static keystat kstat ;
  
  this(char[] name, HINSTANCE hInstance, char[] ClassName) {
    static WNDCLASSEX  winc;
    winc.cbSize         = WNDCLASSEX.sizeof;
    winc.style          = CS_HREDRAW | CS_VREDRAW;
    winc.cbClsExtra     = 0;
    winc.cbWndExtra     = 0;
    winc.hInstance      = hInstance;
    winc.hIcon          = LoadIcon(null , IDI_APPLICATION);
    winc.hCursor        = LoadCursor(null , IDC_ARROW);
    winc.hbrBackground  = cast(HBRUSH)(GetStockObject(WHITE_BRUSH));
    winc.lpszMenuName   = null;
    winc.lpszClassName  = ClassName ;
    super(name, winc) ;
  }

  LRESULT proc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp){
    switch (msg) {
    case WM_DESTROY:
      wglDeleteContext( hRC );
      ReleaseDC(hwnd, WinDC);
      gluw=null ;
      PostQuitMessage(0);
      break ;
    case WM_CREATE:
      WinDC=GetDC(hwnd);
      pixelFormat = ChoosePixelFormat(WinDC,&pfd);
      SetPixelFormat(WinDC,pixelFormat,&pfd);
      hRC=wglCreateContext(WinDC);
      wglMakeCurrent( WinDC, hRC );
      gluw = new GLUW() ;
      gluw.init() ;
      wglMakeCurrent( null, null );
      this.hInstance = (cast(LPCREATESTRUCT)(lp)).hInstance ;
      kstat.init() ;
      break ;
      
    case WM_PAINT:
      static PAINTSTRUCT ps;
      static HDC hdc;

      wglMakeCurrent( WinDC, hRC );
      gluw.display() ;
      SwapBuffers(WinDC);
      wglMakeCurrent( null, null );
      
      hdc = BeginPaint(hwnd, &ps);
      SetBkMode(hdc,TRANSPARENT) ;
      EndPaint(hwnd, &ps);
      break ;

    case WM_SIZE:
      int w=LOWORD(lp) ;
      int h=HIWORD(lp) ;
      
      wglMakeCurrent( WinDC, hRC );
      gluw.reshape(w,h) ;
      wglMakeCurrent( null, null );
      break;

    case WM_LBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_RBUTTONDBLCLK:
    case WM_MOUSEMOVE:
      static POINT point;

      GetCursorPos(&point);
      ScreenToClient( hwnd, &point );

      kstat.mx=point.x ; kstat.my=point.y ;
      
      if( msg==WM_LBUTTONDOWN || msg==WM_RBUTTONDOWN ) SetCapture( hwnd ) ;
      if( msg==WM_LBUTTONUP   || msg==WM_RBUTTONUP   ) ReleaseCapture() ;

      if( msg==WM_MOUSEMOVE || msg==WM_NCMOUSEMOVE ){
      }else{
        kstat.mb=kst.NON ;
        switch( msg ){
        case WM_LBUTTONDOWN:
          kstat.mb|=kst.LBUTTONDOWN ;
          break ;
        case WM_LBUTTONUP:
          kstat.mb|=kst.LBUTTONUP ;
          break ;
        case WM_LBUTTONDBLCLK:
          kstat.mb|=kst.LBUTTONDBLCLK ;
          break ;
        case WM_RBUTTONDOWN:
          kstat.mb|=kst.RBUTTONDOWN ;
          break ;
        case WM_RBUTTONUP:
          kstat.mb|=kst.RBUTTONUP ;
          break ;
        case WM_RBUTTONDBLCLK:
          kstat.mb|=kst.RBUTTONDBLCLK ;
          break ;
        default :
          return (DefWindowProc(hwnd, msg, wp, lp));
          break ;
        }
      }
      if (GetKeyState(VK_SHIFT) < 0){
        kstat.acs |= kst.SHIFT_KEY ;
      }else{
        kstat.acs &= ~(kst.SHIFT_KEY) ;
      }
      if (GetKeyState(VK_CONTROL) < 0){
        kstat.acs |= kst.CNTL_KEY  ;
      }else{
        kstat.acs &= ~(kst.CNTL_KEY)  ;
      }
      if (GetKeyState(VK_MENU) < 0){
        kstat.acs |= kst.ALT_KEY ;
      }else{
        kstat.acs &= ~(kst.ALT_KEY) ;
      }

      if( gluw.mouse(kstat.acs, kstat.mb, kstat.mx, kstat.my )>0 ){
        RedrawWindow( hwnd, NULL, NULL, RDW_INVALIDATE ) ;
      }
      //wglMakeCurrent( WinDC, hRC );
      //wglMakeCurrent( null, null );
      break ;
      
    default :
      return( DefWindowProc(hwnd , msg , wp , lp) );
    }
    return(0);
  }

}
