import std.math;

struct vec2D  {
  double  u ;
  double  v ;
}

struct vec3D  {
  double  x ;
  double  y ;
  double  z ;
  double  q ;

  void set(double d){
    this.x = d ;
    this.y = d ;
    this.z = d ;
  }
  
  void set(vec3D d){
    this.x = d.x ;
    this.y = d.y ;
    this.z = d.z ;
  }
  
  void add(vec3D d){
    this.x = this.x + d.x ;
    this.y = this.y + d.y ;
    this.z = this.z + d.z ;
  }
  
  void add(double d){
    this.x = this.x + d ;
    this.y = this.y + d ;
    this.z = this.z + d ;
  }
  
  void sub(vec3D d){
    this.x = this.x - d.x ;
    this.y = this.y - d.y ;
    this.z = this.z - d.z ;
  }
  
  void sub(double d){
    this.x = this.x - d ;
    this.y = this.y - d ;
    this.z = this.z - d ;
  }
  
  void inv(){
    this.x = -this.x ;
    this.y = -this.y ;
    this.z = -this.z ;
  }
  
  void scale(double d){
    this.x = this.x * d ;
    this.y = this.y * d ;
    this.z = this.z * d ;
  }
  
  int eq( vec3D b ){
    if( this.x == b.x &&
	this.y == b.y &&
	this.z == b.z ) {
      return(1) ;
    }
    return(0) ;
  }
  
  double distance( vec3D b ){
    double lx=this.x-b.x ;
    double ly=this.y-b.y ;
    double lz=this.z-b.z ;
    return( sqrt( lx*lx + ly*ly + lz*lz ) ) ;
  }
  
  double length(){
    return( sqrt( this.x*this.x + this.y*this.y + this.z*this.z ) ) ;
  }
  
  double normalize(){
    double d ;
    d=sqrt( this.x*this.x + this.y*this.y + this.z*this.z ) ;
    if( d==0.0 ){
      this.x=0.0 ;
      this.y=0.0 ;
      this.z=0.0 ;
    }else{
      this.x = this.x/d ;
      this.y = this.y/d ;
      this.z = this.z/d ;
    }
    return( d ) ;
  }

  int rotate( double angle, vec3D vec ){
    double sinval, cosval;
    double m[3][3] ;
    double x,y,z ;
    
    x=vec.x ;
    y=vec.y ;
    z=vec.z ;
    sinval = sin((3.141592654*2)-angle);
    cosval = cos((3.141592654*2)-angle);
    
    m[0][0] = x * x * (1.0 - cosval) + cosval  ;
    m[0][1] = x * y * (1.0 - cosval) - z * sinval;
    m[0][2] = x * z * (1.0 - cosval) + y * sinval;
    
    m[1][0] = x * y * (1.0 - cosval) + z * sinval;
    m[1][1] = y * y * (1.0 - cosval) + cosval  ;
    m[1][2] = y * z * (1.0 - cosval) - x * sinval;
    
    m[2][0] = x * z * (1.0 - cosval) - y * sinval;
    m[2][1] = y * z * (1.0 - cosval) + x * sinval;
    m[2][2] = z * z * (1.0 - cosval) + cosval  ;
    
    
    x=this.x ;
    y=this.y ;
    z=this.z ;
    this.x=m[0][0]*x + m[0][1]*y + m[0][2]*z ;
    this.y=m[1][0]*x + m[1][1]*y + m[1][2]*z ;
    this.z=m[2][0]*x + m[2][1]*y + m[2][2]*z ;
    
    return(0);
  }
  
  double dot( vec3D b ){
    return( (this.x * b.x) + (this.y * b.y) + (this.z * b.z) ) ;
  }
  
  void cross( vec3D b ){
    double tx,ty,tz ;
    tx=(this.y * b.z) - (this.z * b.y) ;
    ty=(this.z * b.x) - (this.x * b.z) ;
    tz=(this.x * b.y) - (this.y * b.x) ;
    this.x=tx ;
    this.y=ty ;
    this.z=tz ;
  }
  
  int LPcross( vec3D eyepos, vec3D ray, vec3D fp0, vec3D fn) {
    double  a, b, t;

    b = ray.dot(fn) ;
    if( b == 0.0 ) return(0);
    
    fp0.sub(eyepos) ;
    a = fp0.dot(fn) ;
    t = a/b ;
    
    this.x = eyepos.x + ray.x * t ;
    this.y = eyepos.y + ray.y * t ;
    this.z = eyepos.z + ray.z * t ;
    
    return(1);
  }
  
  int chkinside( vec3D pos, vec3D fp0, vec3D fp1, vec3D fp2 ) {
    vec3D   n0 ;
    vec3D   n1 ;
    vec3D   n2 ;
    double  res0,res1,res2;
    
    fp0.sub(pos) ;
    fp1.sub(pos) ;
    fp2.sub(pos) ;
    n0=fp0 ;
    n1=fp1 ;
    n2=fp2 ;
    n0.cross(fp1) ;
    n1.cross(fp2) ;
    n2.cross(fp0) ;
    res0=n0.dot(n1) ;
    res1=n1.dot(n2) ;
    res2=n2.dot(n0) ;
    res0 = res0<0.0 ? -1.0 : 1.0 ;
    res1 = res1<0.0 ? -1.0 : 1.0 ;
    res2 = res2<0.0 ? -1.0 : 1.0 ;
    
    if( res0==res1 && res1==res2 && res2==res0 ){
      return(1) ;
    }
    return(0) ;
  }

  void print()
  {
    //printf("x=%f,y=%f\n",x,y) ;
  }
}

struct face3D {
  int    p0 ;
  int    p1 ;
  int    p2 ;
  int    p3 ;
}

//class vec3D {
//  double  x, y, z, q ;
//
//  this(){
//  }
//  
//  int add( vec3D a ){
//    printf("aaa\n") ;
//    return(0) ;
//  }
//}
